'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _GitCommit = require('../models/GitCommit');

var _GitCommit2 = _interopRequireDefault(_GitCommit);

var _GitTree = require('../models/GitTree');

var _GitTree2 = _interopRequireDefault(_GitTree);

var _GitObjectManager = require('../managers/GitObjectManager');

var _GitObjectManager2 = _interopRequireDefault(_GitObjectManager);

var _write = require('../utils/write');

var _write2 = _interopRequireDefault(_write);

var _resolveRef = require('../utils/resolveRef');

var _resolveRef2 = _interopRequireDefault(_resolveRef);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function writeTreeToDisk({ gitdir, dirpath, tree }) {
  for (let entry of tree) {
    let { type, object } = await _GitObjectManager2.default.read({
      gitdir,
      oid: entry.oid
    });
    let entrypath = `${dirpath}/${entry.path}`;
    switch (type) {
      case 'blob':
        await (0, _write2.default)(entrypath, object);
        break;
      case 'tree':
        let tree = _GitTree2.default.from(object);
        await writeTreeToDisk({ gitdir, dirpath: entrypath, tree });
        break;
      default:
        throw new Error(`Unexpected object type ${type} found in tree for '${dirpath}'`);
    }
  }
}

exports.default = async function checkout({ workdir, gitdir, remote, ref }) {
  // Get tree oid
  let oid;
  try {
    oid = await (0, _resolveRef2.default)({ gitdir, ref });
  } catch (e) {
    oid = await (0, _resolveRef2.default)({ gitdir, ref: `${remote}/${ref}` });
    await (0, _write2.default)(`${gitdir}/refs/heads/${ref}`, oid + '\n');
  }
  let commit = await _GitObjectManager2.default.read({ gitdir, oid });
  if (commit.type !== 'commit') {
    throw new Error(`Unexpected type: ${commit.type}`);
  }
  let comm = _GitCommit2.default.from(commit.object.toString('utf8'));
  let sha = comm.headers().tree;
  // Get top-level tree
  let { type, object } = await _GitObjectManager2.default.read({ gitdir, oid: sha });
  if (type !== 'tree') throw new Error(`Unexpected type: ${type}`);
  let tree = _GitTree2.default.from(object);
  // Write files. TODO: Write them atomically
  await writeTreeToDisk({ gitdir, dirpath: workdir, tree });
  // Update HEAD TODO: Handle non-branch cases
  (0, _write2.default)(`${gitdir}/HEAD`, `ref: refs/heads/${ref}`);
};
//# sourceMappingURL=checkout.js.map