'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _GitCommit = require('../models/GitCommit');

var _GitCommit2 = _interopRequireDefault(_GitCommit);

var _resolveRef = require('../utils/resolveRef');

var _resolveRef2 = _interopRequireDefault(_resolveRef);

var _GitObjectManager = require('../managers/GitObjectManager');

var _GitObjectManager2 = _interopRequireDefault(_GitObjectManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = async function listCommits({
  gitdir,
  start,
  finish /*: {
         gitdir: string,
         start: Array<string>,
         finish: Array<string>
         } */
}) {
  let startingSet = new Set();
  let finishingSet = new Set();
  for (let ref of start) {
    startingSet.add((await (0, _resolveRef2.default)({ gitdir, ref })));
  }
  for (let ref of finish) {
    // We may not have these refs locally so we must try/catch
    try {
      let oid = await (0, _resolveRef2.default)({ gitdir, ref });
      finishingSet.add(oid);
    } catch (err) {}
  }
  let visited = new Set(); /*: Set<string> */

  // Because git commits are named by their hash, there is no
  // way to construct a cycle. Therefore we won't worry about
  // setting a default recursion limit.
  async function walk(oid) {
    visited.add(oid);
    let { type, object } = await _GitObjectManager2.default.read({ gitdir, oid });
    (0, _assert2.default)(type === 'commit');
    let commit = _GitCommit2.default.from(object);
    let parents = commit.headers().parent;
    for (oid of parents) {
      if (!finishingSet.has(oid) && !visited.has(oid)) {
        await walk(oid);
      }
    }
  }

  // Let's go walking!
  for (let oid of startingSet) {
    await walk(oid);
  }
  return visited;
};
//# sourceMappingURL=listCommits.js.map