'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

var _GitRemoteHTTP = require('../managers/GitRemoteHTTP');

var _GitRemoteHTTP2 = _interopRequireDefault(_GitRemoteHTTP);

var _listCommits = require('./listCommits');

var _listCommits2 = _interopRequireDefault(_listCommits);

var _listObjects = require('./listObjects');

var _listObjects2 = _interopRequireDefault(_listObjects);

var _packObjects = require('./pack-objects');

var _packObjects2 = _interopRequireDefault(_packObjects);

var _resolveRef = require('../utils/resolveRef');

var _resolveRef2 = _interopRequireDefault(_resolveRef);

var _pktLineEncoder = require('../utils/pkt-line-encoder');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = async function push({ gitdir, ref = 'HEAD', url, auth }) {
  let oid = await (0, _resolveRef2.default)({ gitdir, ref });
  let remote = new _GitRemoteHTTP2.default(url);
  remote.auth = auth;
  await remote.preparePush();
  let commits = await (0, _listCommits2.default)({
    gitdir,
    start: [oid],
    finish: remote.refs.values()
  });
  let objects = await (0, _listObjects2.default)({ gitdir, oids: commits });
  let packstream = new _stream2.default.PassThrough();
  let oldoid = remote.refs.get(ref) || '0000000000000000000000000000000000000000';
  packstream.write((0, _pktLineEncoder.encode)(`${oldoid} ${oid} ${ref}\0 report-status\n`));
  packstream.write((0, _pktLineEncoder.flush)());
  (0, _packObjects2.default)({
    gitdir,
    oids: [...objects],
    outputStream: packstream
  });
  let response = await remote.push(packstream);
  return response;
};
//# sourceMappingURL=push.js.map