'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _GitCommit = require('../models/GitCommit');

var _GitCommit2 = _interopRequireDefault(_GitCommit);

var _GitObjectManager = require('../managers/GitObjectManager');

var _GitObjectManager2 = _interopRequireDefault(_GitObjectManager);

var _resolveRef = require('../utils/resolveRef');

var _resolveRef2 = _interopRequireDefault(_resolveRef);

var _openpgp = require('openpgp');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const HttpKeyServer = new _openpgp.HKP();

exports.default = async function verify({ gitdir, ref, publicKeys }) {
  const oid = await (0, _resolveRef2.default)({ gitdir, ref });
  const { type, object } = await _GitObjectManager2.default.read({ gitdir, oid });
  if (type !== 'commit') {
    throw new Error(`git.verify() was expecting a ref type 'commit' but got type '${type}'`);
  }
  let commit = _GitCommit2.default.from(object);
  let author = commit.headers().author;
  let keys = await commit.listSigningKeys();
  if (!publicKeys) {
    let keyArray = await Promise.all(keys.map(id => HttpKeyServer.lookup({ keyId: id })));
    publicKeys = keyArray.join('\n');
  }
  let validity = await commit.verify(publicKeys);
  if (!validity) return false;
  return { author, keys };
};
//# sourceMappingURL=verify.js.map