'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _read = require('../utils/read');

var _read2 = _interopRequireDefault(_read);

var _write = require('../utils/write');

var _write2 = _interopRequireDefault(_write);

var _GitIndex = require('../models/GitIndex');

var _GitIndex2 = _interopRequireDefault(_GitIndex);

var _asyncLock = require('async-lock');

var _asyncLock2 = _interopRequireDefault(_asyncLock);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import Lock from '../utils/lockfile'

// TODO: replace with an LRU cache?

// import LockManager from 'travix-lock-manager'
const map /*: Map<string, GitIndex> */ = new Map();
// const lm = new LockManager()
const lock = new _asyncLock2.default();

class GitIndexManager {
  static async acquire(filepath, closure) {
    await lock.acquire(filepath, async function () {
      let index = map.get(filepath);
      if (index === undefined) {
        // Acquire a file lock while we're reading the index
        // to make sure other processes aren't writing to it
        // simultaneously, which could result in a corrupted index.
        // const fileLock = await Lock(filepath)
        const rawIndexFile = await (0, _read2.default)(filepath);
        index = _GitIndex2.default.from(rawIndexFile);
        // cache the GitIndex object so we don't need to re-read it
        // every time.
        // TODO: save the stat data for the index so we know whether
        // the cached file is stale (modified by an outside process).
        map.set(filepath, index);
        // await fileLock.cancel()
      }
      await closure(index);
      if (index._dirty) {
        // Acquire a file lock while we're writing the index file
        // let fileLock = await Lock(filepath)
        const buffer = index.toObject();
        await (0, _write2.default)(filepath, buffer);
        index._dirty = false;
      }
      // For now, discard our cached object so that external index
      // manipulation is picked up. TODO: use lstat and compare
      // file times to determine if our cached object should be
      // discarded.
      map.delete(filepath);
    });
  }
}
exports.default = GitIndexManager;
//# sourceMappingURL=GitIndexManager.js.map