'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lock = lock;
exports.unlock = unlock;

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _sleep = require('./sleep');

var _sleep2 = _interopRequireDefault(_sleep);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const delayedReleases = new Map();
// This is modeled after the lockfile strategy used by the git source code.


const mkdir = (0, _pify2.default)(_fs2.default.mkdir);
const rmdir = (0, _pify2.default)(_fs2.default.rmdir);

async function lock(filename /*: string */
, triesLeft /*: number */ = 3) {
  // check to see if we still have it
  if (delayedReleases.has(filename)) {
    clearTimeout(delayedReleases.get(filename));
    delayedReleases.delete(filename);
    return;
  }
  if (triesLeft === 0) {
    throw new Error(`Unable to acquire lockfile '${filename}'. Exhausted tries.`);
  }
  try {
    await mkdir(`${filename}.lock`);
  } catch (err) {
    if (err.code === 'EEXIST') {
      await (0, _sleep2.default)(100);
      await lock(filename, triesLeft - 1);
    }
  }
}

async function unlock(filename /*: string */
, delayRelease /*: number */ = 50) {
  if (delayedReleases.has(filename)) {
    throw new Error('Cannot double-release lockfile');
  }
  // Basically, we lie and say it was deleted ASAP.
  // But really we wait a bit to see if you want to acquire it again.
  delayedReleases.set(filename, setTimeout(async () => {
    delayedReleases.delete(filename);
    await rmdir(`${filename}.lock`);
  }));
}
//# sourceMappingURL=lockfile.js.map