'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _exists = require('./exists');

var _exists2 = _interopRequireDefault(_exists);

var _read = require('./read');

var _read2 = _interopRequireDefault(_read);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = async function resolveRef({ gitdir, ref, depth }) {
  if (depth !== undefined) {
    depth--;
    if (depth === -1) {
      return ref;
    }
  }
  let sha;
  // Is it a ref pointer?
  if (ref.startsWith('ref: ')) {
    ref = ref.slice('ref: '.length);
    return resolveRef({ gitdir, ref, depth });
  }
  // Is it a complete and valid SHA?
  if (ref.length === 40) {
    if (await (0, _exists2.default)(`${gitdir}/objects/${ref.slice(0, 2)}/${ref.slice(2)}`)) {
      return ref;
    }
  }
  // Is it a special ref?
  if (ref === 'HEAD' || ref === 'MERGE_HEAD') {
    sha = await (0, _read2.default)(`${gitdir}/${ref}`, { encoding: 'utf8' });
    if (sha) return resolveRef({ gitdir, ref: sha.trim(), depth });
  }
  // Is it a full ref?
  if (ref.startsWith('refs/')) {
    sha = await (0, _read2.default)(`${gitdir}/${ref}`, { encoding: 'utf8' });
    if (sha) return resolveRef({ gitdir, ref: sha.trim(), depth });
  }
  // Is it a (local) branch?
  sha = await (0, _read2.default)(`${gitdir}/refs/heads/${ref}`, { encoding: 'utf8' });
  if (sha) return resolveRef({ gitdir, ref: sha.trim(), depth });
  // Is it a tag?
  sha = await (0, _read2.default)(`${gitdir}/refs/tags/${ref}`, { encoding: 'utf8' });
  if (sha) return resolveRef({ gitdir, ref: sha.trim(), depth });
  // Is it remote branch?
  sha = await (0, _read2.default)(`${gitdir}/refs/remotes/${ref}`, { encoding: 'utf8' });
  if (sha) return resolveRef({ gitdir, ref: sha.trim(), depth });
  // Do we give up?
  throw new Error(`Could not resolve reference ${ref}`);
};
//# sourceMappingURL=resolveRef.js.map